# -*- coding: utf-8 -*-
__all__ = ["load_plugin_modules"]
import pkgutil
import importlib
import pkg_resources
import functools
import kanjidb.builder.plugins


def load_plugin_modules(imports, names):
    def iter_namespace(ns_pkg):
        for finder, name, ispkg in pkgutil.iter_modules(
            ns_pkg.__path__, ns_pkg.__name__ + "."
        ):
            yield name, functools.partial(importlib.import_module, name)
        for _ in pkg_resources.iter_entry_points("kanjidb.builder.plugins"):
            yield "kanjidb.builder.plugins.{}".format(_.name), _.load

    def normalize(name):
        return name[name.rfind(".") + 1 :]

    modules = {
        normalize(name): load()
        for name, load in iter_namespace(kanjidb.builder.plugins)
        if normalize(name) in names
    }

    modules.update(
        {
            _["name"]: importlib.import_module(_["path"])
            for _ in imports
            if _["name"] in names
        }
    )

    return modules



__all__ = ["Plugin", "run", "load"]
from kanjidb.builder.plugins import PluginBase, kanjistream, jsonstream

try:
    from jamdict.kanjidic2 import Kanjidic2XMLParser
except Exception as e:
    raise Exception("kanjidic2 requires jamdict to be installed") from e


class Plugin(PluginBase):
    @property
    def template_config(self):
        return {
            "inputs": [{"type": "var", "name": "kanjis"}],
            "outputs": [{"type": "var", "name": "db"}],
            "kd2_file": "kd2.xml",
        }

    @property
    def required_config(self):
        return self.template_config

    def __call__(self, **kwargs):
        """Fill database with Kanjidic2 infos.
        """
        run(
            inputs=self.plugin_config["inputs"],
            outputs=self.plugin_config["outputs"],
            kd2_file=self.plugin_config["kd2_file"],
            kwargs=kwargs,
        )

        return kwargs

    def __repr__(self):
        return "Kanjidic2"


def run(inputs, outputs=None, *, kd2_file, kwargs=None):
    """Produce a JSON dict containing kanjis data from a Kanjidic2 XML file
    (`download here <http://www.edrdg.org/wiki/index.php/KANJIDIC_Project>`_).

    Kanjidic2 contains the whole dictionary, but only kanjis read from
    input streams are generated.

    Configuration:

    .. code-block:: yaml

        run:
        - kanjidic2:
          kd2_file: string
          inputs:
          - type: stream|var
            [separator: string] # stream only
            [encoding: string] # stream only
            [path: string] # stream only
            [name: string] # var only
          outputs:
          - type: stream|var
            [path: string] # stream only
            [indent: int] # stream only
            [name: string] # var only

    Generate data for kanjis read from stdin:

    .. code-block:: yaml

        run:
        - kanjidic2:
          kd2_file: path/to/kanjidic2.xml
          inputs:
          - type: stream
            encoding: utf8
            separator: "\\n"
            path: "-"
          outputs:
          - type: var
            name: result

    Python equivalent:

    .. code-block:: python

        from kanjidb.builder.plugins import kanjidic2

        result = kanjidic2.run(
            kd2_file="path/to/kanjidic2.xml",
            inputs=[{
                "type": "stream",
                "encoding": "utf8",
                "separator": "\\n",
                "path": "-"
            }]
        )

    Read kanjis from file and export generated data to file:

    .. code-block:: yaml

        run:
        - kanjidic2:
          kd2_file: path/to/kanjidic2.xml
          inputs:
          - type: stream
            encoding: utf8
            separator: "\\n"
            path: path/to/kanjis.txt
          outputs:
          - type: stream
            indent: 4
            path: path/to/db.json

    Python equivalent:

    .. code-block:: python

        from kanjidb.builder.plugins import kanjidic2

        kanjidic2.run(
            kd2_file="path/to/kanjidic2.xml",
            inputs=[{
                "type": "stream",
                "encoding": "utf8",
                "separator": "\\n",
                "path": "path/to/kanjis.txt"
            }],
            outputs=[{
                "type": "stream",
                "indent": 4,
                "path": "path/to/db.json"
            }]
        )

    :param inputs: input streams
    :param output: output streams
    :param kwargs: context
    :return: a JSON object with kanjis data
    """
    outputs = outputs if outputs is not None else []
    kwargs = kwargs if kwargs is not None else {}

    # Read kanjis to generate
    kanjis = kanjistream.run(inputs=inputs, kwargs=kwargs)

    # Load external Kanjidic2 XML file
    data = load(kd2_file)
    data = {_.literal: _ for _ in data.characters}
    data = {_: data[_].to_json() for _ in kanjis}

    # Write to output streams
    jsonstream.run(
        inputs=[{"type": "var", "name": "data"}],
        outputs=outputs,
        kwargs={"data": data},
    )

    # Store to variables
    for _ in outputs:
        if _["type"] == "var":
            kwargs[_["name"]] = data

    return data


def load(stream):
    """Load a Kanjidic2 XML file using `jamdict`.

    :param stream: filelike object or filename
    :return: data loaded with `jamdict`.
    """
    parser = Kanjidic2XMLParser()
    if hasattr(stream, "read"):
        return parser.parse_str(stream.read())
    else:
        return parser.parse_file(stream)



__all__ = ["Configuration"]
import os
import sys
import yaml
from kanjidb import loader


DEFAULT_PLUGINS = ["kanjistream", "kanjidic2", "jsonwriter"]

DEFAULT_CONFIG_YML = "config.yml"


class Configuration:
    def __init__(self, *, verbose=None):
        self.run = []
        self.verbose = verbose

    def template_config(self):
        """Build the default configuration.

        This is for first use of KanjiDB to initialize
        a YAML configuration file.

        Generate a configuration with all default parameters
        for default active plugins (see `DEFAULT_PLUGINS`).

        :return: default JSON config dict
        """
        modules = loader.load_plugin_modules(DEFAULT_PLUGINS)

        return {
            "run": [{_: modules[_].Plugin().template_config} for _ in DEFAULT_PLUGINS]
        }

    def load(self, stream=None, *, default=None):
        """Load configuration from YAML input.

        :param stream: filelike or str object
        :param default: default filename
        """
        default = default if default is not None else DEFAULT_CONFIG_YML

        if stream == "-":
            stream = sys.stdin

        # Filelike object
        if hasattr(stream, "read"):
            config = yaml.safe_load(stream)
        # If invalid filename
        elif not os.path.isfile(stream):
            if not default:
                raise FileNotFoundError(stream)

            # Generate configuration file for first time
            config = self.template_config()
            with open(default, "w+") as f:
                yaml.safe_dump(config, stream=f)
        # If valid filename
        else:
            with open(stream, "r") as f:
                config = yaml.safe_load(f)

        self.run = self._load_steps(
            imports=config.get("import", []), steps=config["run"]
        )

        if self.verbose:
            print("Configuration loaded")

    def _load_steps(self, imports, steps):
        result = []

        for step in steps:
            modules = loader.load_plugin_modules(imports, step.keys())
            for name, config in step.items():
                plugin = modules[name].Plugin()
                result.append(plugin)

                c = plugin.template_config
                c.update(plugin.required_config)
                c.update(config)

                plugin.configure(global_config=self, plugin_config=c)

        return result



__all__ = ["build", "main"]
from kanjidb.builder.configuration import Configuration


def build(config):
    """Build a kanji database.

    :param config: builder configuration
    :return: build result
    """
    kwargs = {}

    for step in config.run:
        if config.verbose:
            print("Running {}...".format(step))

        new_kwargs = step(**kwargs)

        if new_kwargs is not None:
            kwargs = new_kwargs

    return kwargs


def main(argv):
    import argparse

    parser = argparse.ArgumentParser(
        prog="KanjiDB builder", description="Kanji database builder"
    )
    parser.add_argument("-v", "--verbose", action="store_true", help="Verbosity level")
    parser.add_argument("config", nargs="?", help="YAML configuration file")
    args = parser.parse_args(argv)

    config = Configuration(verbose=args.verbose)
    config.load(args.config, default=args.config)

    build(config)



import json
from aiohttp import web
import aiohttp_cors
import aiohttp_swagger
from typing import Callable, Any, List


def validate_required_params(_fun=None, *, names):
    """Validate that a query has all required parameters.

    Role of this decorator is to force returning a `web.HTTPForbidden`
    with an explicit message when one of required query parameters
    is missing.

    This will call the wrapped function with a dict containing
    all required parameters values.

    :param names: list of required parameters
    :return: result of wrapped function or `web.HTTPForbidden`
    """

    def wrapper(fun):
        async def run(self: web.View, *args, **kwargs):
            # Note that `self` is a `web.View` object.
            query = self.request.rel_url.query
            # Decode POST parameters
            if self.request.body_exists and self.request.content_type.endswith("json"):
                data = await self.request.json()
            else:
                data = {}
            # Check and get all parameters
            vals = {}
            for name in names:
                val = data.get(name, None) or query.get(name, None)
                if not val:
                    raise web.HTTPForbidden(
                        reason="{} parameter is required".format(name)
                    )
                vals[name] = val
            # Forward parameters to wrapped functions
            return await fun(self, *args, required_params=vals, **kwargs)

        return run

    return wrapper if not _fun else wrapper(_fun)


def KanjiView(*, db) -> web.View:
    class Wrapper(web.View, aiohttp_cors.CorsViewMixin):
        async def get(self):
            kanji = self.request.match_info["kanji"]
            data = db.get(kanji, None)
            if not data:
                return web.HTTPNotFound(text=json.dumps({"result": "Ko"}))

            data["kanji"] = kanji
            return web.Response(
                text=json.dumps({"result": "Ok", "params": data}, ensure_ascii=False)
            )

    return Wrapper


def KanjiListView(*, db) -> web.View:
    class Wrapper(web.View, aiohttp_cors.CorsViewMixin):
        async def get(self):
            return web.Response(
                text=json.dumps(
                    {"result": "Ok", "params": list(db.keys())}, ensure_ascii=False
                )
            )

    return Wrapper


def Application(
    *args, db, swagger_yml: str, swagger_url: str = None, base_url: str = None, **kwargs
):
    app = web.Application(*args, **kwargs)

    base_url = base_url or ""
    cors = aiohttp_cors.setup(
        app,
        defaults={
            "*": aiohttp_cors.ResourceOptions(
                allow_credentials=True, expose_headers="*", allow_headers="*",
            )
        },
    )
    cors.add(app.router.add_view(base_url + "/kanji", KanjiListView(db=db)))
    cors.add(app.router.add_view(base_url + "/kanji/{kanji}", KanjiView(db=db)))

    aiohttp_swagger.setup_swagger(
        app, swagger_from_file=swagger_yml, swagger_url=swagger_url
    )

    return app



# -*- coding: utf-8 -*-
__all__ = ["Plugin", "run", "loads", "load", "dump"]
import sys
import os
import functools
import re
from kanjidb.builder.plugins import PluginBase
import kanjidb.encoding


class Plugin(PluginBase):
    @property
    def template_config(self):
        return {
            "inputs": [
                {
                    "type": "stream",
                    "separator": os.linesep,
                    "encoding": kanjidb.encoding.UTF8,
                    "path": "-",
                }
            ],
            "outputs": [{"type": "var", "name": "result"}],
        }

    @property
    def required_config(self):
        config = self.template_config

        return config

    def __call__(self, **kwargs):
        run(
            inputs=self.plugin_config["inputs"],
            outputs=self.plugin_config["outputs"],
            kwargs=kwargs,
        )

        return kwargs

    def __repr__(self):
        return "KanjiStream"


def run(inputs, outputs=None, *, kwargs=None):
    """Read/write kanjis from/to streams.

    Given an input stream containing one unicode encoded kanji
    per line such as:

    .. code-block::

        U+4E00
        U+4E8C
        ...

    This plugin will produce a list containing:

    .. code-block::

        ["一", "二", ...]

    Configuration:

    .. code-block:: yaml

        run:
        - kanjistream:
          inputs:
          - type: stream|var
            [separator: string] # stream only
            [encoding: string] # stream only
            [path: string] # stream only
            [name: string] # var only
          outputs:
          - type: stream|var
            [separator: string] # stream only
            [encoding: string] # stream only
            [path: string] # stream only
            [name: string] # var only

    Read from stdin and store to `result`:

    .. code-block:: yaml

        run:
        - kanjistream:
          inputs:
          - type: stream
            encoding: utf8
            separator: "\\n"
            path: "-"
          outputs:
          - type: var
            name: result

    Python equivalent:

    .. code-block:: python

        from kanjidb.builder.plugins import kanjistream

        result = kanjistream.run(
            inputs=[{
                "type": "stream",
                "encoding": "utf8",
                "separator": "\\n",
                "path": "-"
            }]
        )

    Get from `result` and write to stdout:

    .. code-block:: yaml

        run:
        - kanjistream:
          inputs:
          - type: var
            name: result
          outputs:
          - type: stream
            encoding: utf8
            separator: "\\n"
            path: "-"

    Python equivalent:

    .. code-block:: python

        from kanjidb.builder.plugins import kanjistream

        kanjistream.run(
            inputs=[{
                "type": "var",
                "name": "result"
            }],
            outputs=[{
                "type": "stream",
                "encoding": "utf8",
                "separator": "\\n",
                "path": "-"
            }],
            kwargs={"result": ["一", "二"]}
        )

    :param inputs: input streams
    :param output: output streams
    :param kwargs: context
    :return: a list of kanjis read from input streams
    """
    outputs = outputs if outputs is not None else []
    kwargs = kwargs if kwargs is not None else {}

    kanjis = []

    # Read from inputs
    for _ in inputs:
        if _["type"] == "stream":
            in_kanjis = load(_["path"], sep=_.get("separator", None))
        elif _["type"] == "var":
            in_kanjis = kwargs[_["name"]]
        else:
            raise Exception("Invalid input {}".format(_["type"]))

        kanjis += kanjidb.encoding.decode_all(
            in_kanjis, encoding=_.get("encoding", None)
        )

    # Write to outputs
    for _ in outputs:
        out_kanjis = kanjidb.encoding.encode_all(kanjis, encoding=_.get("encoding"))

        if _["type"] == "stream":
            dump(out_kanjis, _["path"], sep=_.get("separator", None))
        elif _["type"] == "var":
            kwargs[_["name"]] = out_kanjis
        else:
            raise Exception("Invalid output {}".format(_["type"]))

    return kanjis


def loads(s, *, encoding=None, decode=None, sep=None):
    """Load a list of kanjis from string.

    Usage example:

    .. code-block:: python

        >> loader.loads("一二")
        ['一', '二']

    Using custom separator:

    .. code-block:: python

        >> loader.loads("一;二"), sep=";")
        ['一', '二']

    Default behaviour is that kanjis must be UTF-8 encoded, but
    this can customized:

    .. code-block:: python

        >> loader.loads("U+4E00", encoding=encoding.UNICODE_PLUS)
        ['一']

        >> loader.loads("\u4E00", encoding=encoding.UNICODE_ESCAPE)
        ['一']

    :param s: string to decode
    :param encoding: kanjis encoding
    :param decode: how to decode kanjis
    :param sep: separator between kanjis
    :return: list of decoded kanjis
    """
    decode = (
        decode
        if decode is not None
        else functools.partial(kanjidb.encoding.decode, encoding=encoding)
    )

    symbols = s if sep is None else s.split(sep)
    symbols = [re.sub("[ \t\r\n]", "", _) for _ in symbols]

    return [decode(_) for _ in symbols]


def load(*streams, encoding=None, decode=None, sep=None):
    """Load a list of kanjis from input streams.

    Usage example:

    .. code-block:: python

        load_kanjis("a.txt", "b.txt")

    Using filelike objects:

    .. code-block:: python

        load_kanjis(sys.stdin)

        with open("a.txt", "rb") as f:
            load_kanjis(f)

    Default kanjis separator is the newline character, but this can
    be customized as such:

    .. code-block:: python

        load_kanjis("a.txt", "b.txt", sep=";")

    :param streams: list of input streams to read
    :param encoding: kanjis encoding
    :param decode: how to decode kanjis
    :param sep: separator between kanjis
    :return: list of decoded kanjis
    """
    sep = sep if sep is not None else os.linesep

    def wrapper():
        for stream in streams:
            # stdin
            if stream == "-":
                stream = sys.stdin
            # Filelike object
            if hasattr(stream, "read"):
                content = stream.read()
            # Filename
            else:
                with open(stream, "rb") as f:
                    content = f.read().decode()

            yield from loads(content, encoding=encoding, decode=decode, sep=sep)

    return list(wrapper())


def dump(kanjis, *streams, encoding=None, encode=None, sep=None):
    sep = sep if sep is not None else os.linesep
    encode = (
        encode
        if encode is not None
        else functools.partial(kanjidb.encoding.encode, encoding=encoding)
    )

    content = sep.join([encode(_) for _ in kanjis])

    for stream in streams:
        # stdout
        if stream == "-":
            stream = sys.stdout.buffer
        # Filelike object
        if hasattr(stream, "write"):
            stream.write(content.encode())
        # Filename
        else:
            with open(stream, "wb") as f:
                f.write(content.encode())



# -*- coding: utf-8 -*-
__all__ = ["JSONStreamTestCase"]
import sys
import unittest
from kanjidb.builder.plugins import jsonstream

UTF8_OUTPUT = """{
"一": {},
"二": {}
}"""


class JSONStreamTestCase(unittest.TestCase):
    def test_dump(self):
        db = {"一": {}, "二": {}}
        jsonstream.dump(db, "-", sort_keys=True)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
__all__ = ["PluginsHelloWorldTestCase"]
import os
import unittest
from kanjidb import builder
from kanjidb.builder.plugins import helloworld
import kanjidb.encoding

DATA_DIR = os.path.join(os.path.dirname(__file__), "data")
SAMPLE_TXT = os.path.join(DATA_DIR, "sample_helloworld.yml")


class PluginsHelloWorldTestCase(unittest.TestCase):
    def test_plugin(self):
        """Running via Plugin interface.
        """
        plugin = helloworld.Plugin()
        config = plugin.required_config
        config.update(
            {
                "outputs": [
                    {
                        "type": "stream",
                        "encoding": kanjidb.encoding.UNICODE_ESCAPE,
                        "path": "-",
                    }
                ]
            }
        )

        plugin.configure(global_config={}, plugin_config=config)

        plugin()

    def test_run(self):
        """Running via code.
        """
        helloworld.run()

    def test_sample(self):
        builder.main([SAMPLE_TXT])


if __name__ == "__main__":
    unittest.main()



__all__ = ["PluginBase"]
from abc import ABC, abstractmethod


class PluginBase(ABC):
    @property
    def template_config(self):
        """Return the template config for this plugin.

        This config will appear in default generated file.

        :return: a dict containing template config
        """
        return {}

    @property
    def required_config(self):
        """Return the required config for this plugin.

        This config will be merged with the loaded one.

        :return: a dict containing required config
        """
        return {}

    def configure(self, *, global_config, plugin_config):
        """Configure this plugin.

        Parameter `plugin_config` is the combination of loaded
        config and `required_config`, ensuring all parameters
        required to run this plugin are present.

        :param global_config: global builder configuration
        :param plugin_config: plugin configuration
        """
        self.global_config = global_config
        self.plugin_config = plugin_config

    @abstractmethod
    def __call__(self, **kwargs):
        """Execute this plugin.

        :param kwargs: inputs
        :return: outputs
        """
        raise NotImplementedError()



# -*- coding: utf-8 -*-
__all__ = ["EncodingTestCase"]
import unittest
from kanjidb.encoding import decode, encode, UNICODE_ESCAPE, UNICODE_PLUS, UTF8


class EncodingTestCase(unittest.TestCase):
    def test_decode(self):
        self.assertEqual(decode("U+4e00", encoding=UNICODE_PLUS), "一")
        self.assertEqual(decode("U4e00", encoding=UNICODE_PLUS), "一")
        self.assertEqual(decode(r"\u4e00", encoding=UNICODE_ESCAPE), "一")
        self.assertEqual(decode("一", encoding=UTF8), "一")
        self.assertEqual(decode("一"), "一")

    def test_encode(self):
        self.assertEqual(encode("一", encoding=UNICODE_PLUS), "U+4e00")
        self.assertEqual(encode("一", encoding=UNICODE_PLUS, prefix="U"), "U4e00")
        self.assertEqual(encode("一", encoding=UNICODE_ESCAPE), r"\u4e00")
        self.assertEqual(encode("一", encoding=UTF8), "一")
        self.assertEqual(encode("一"), "一")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
__all__ = ["PluginsKanjidic2TestCase"]
import os
import unittest
from kanjidb import builder
from kanjidb.builder.plugins import kanjistream, kanjidic2

DATA_DIR = os.path.join(os.path.dirname(__file__), "data")
KANJIDIC2_XML = os.path.join(DATA_DIR, "kanjidic2.xml")
KANJIS_UTF8_TXT = os.path.join(DATA_DIR, "kanjis_utf8.txt")
SAMPLE_TXT = os.path.join(DATA_DIR, "sample_kanjidic2.yml")


class PluginsKanjidic2TestCase(unittest.TestCase):
    def test_plugin(self):
        plugin = kanjidic2.Plugin()
        config = plugin.required_config
        config.update(
            {
                "inputs": [{"type": "var", "name": "kanjis"}],
                "outputs": [{"type": "var", "name": "db"}],
                "kd2_file": KANJIDIC2_XML,
            }
        )
        plugin.configure(global_config={}, plugin_config=config)

        result = plugin(kanjis=kanjistream.load(KANJIS_UTF8_TXT, sep=";"))
        print(result)
        self.assertTrue("db" in result, "Invalid output")
        db = result["db"]
        self.assertTrue(len(db) == 2, "Invalid result")

    def test_run(self):
        db = kanjidic2.run(
            inputs=[{"type": "var", "name": "kanjis"}],
            outputs=[{"type": "var", "name": "db"}],
            kd2_file=KANJIDIC2_XML,
            kwargs={"kanjis": kanjistream.load(KANJIS_UTF8_TXT, sep=";")},
        )

        self.assertTrue(len(db) == 2, "Invalid result")

    def test_sample(self):
        builder.main([SAMPLE_TXT])


if __name__ == "__main__":
    unittest.main()



__all__ = ["Plugin", "load"]
import os
from kanjidb.builder.plugins import PluginBase
from kanjidb import encoding


class Plugin(PluginBase):
    @property
    def template_config(self):
        return {"sources": "vectorials/"}

    @property
    def required_config(self):
        config = self.template_config
        config.update({"input": {"type": "var", "name": "db"}})

        return config

    def __call__(self, **kwargs):
        """Fill database with Kanjidic2 infos.

        :param db: database
        """
        run(input=self.plugin_config["input"], kwargs=kwargs)

        return kwargs

    def __repr__(self):
        return "KanjiVG"


def run(input, *, kwargs):
    """Produce a JSON dict containing kanjis data from a Kanjidic2 XML file
    (`download here <http://www.edrdg.org/wiki/index.php/KANJIDIC_Project>`_).

    Kanjidic2 contains the whole dictionary, but only kanjis read from
    input streams are generated.

    Configuration:

    .. code-block:: yaml

        run:
        - kanjidic2:
          kd2_file: string
          inputs:
          - type: stream|var
            [separator: string] # stream only
            [encoding: string] # stream only
            [path: string] # stream only
            [name: string] # var only
          outputs:
          - type: stream|var
            [path: string] # stream only
            [indent: int] # stream only
            [name: string] # var only

    Generate data for kanjis read from stdin:

    .. code-block:: yaml

        run:
        - kanjidic2:
          kd2_file: path/to/kanjidic2.xml
          inputs:
          - type: stream
            encoding: utf8
            separator: "\\n"
            path: "-"
          outputs:
          - type: var
            name: result

    Python equivalent:

    .. code-block:: python

        from kanjidb.builder.plugins import kanjidic2

        result = kanjidic2.run(
            kd2_file="path/to/kanjidic2.xml",
            inputs=[{
                "type": "stream",
                "encoding": "utf8",
                "separator": "\\n",
                "path": "-"
            }]
        )

    Read kanjis from file and export generated data to file:

    .. code-block:: yaml

        run:
        - kanjidic2:
          kd2_file: path/to/kanjidic2.xml
          inputs:
          - type: stream
            encoding: utf8
            separator: "\\n"
            path: path/to/kanjis.txt
          outputs:
          - type: stream
            indent: 4
            path: path/to/db.json

    Python equivalent:

    .. code-block:: python

        from kanjidb.builder.plugins import kanjidic2

        kanjidic2.run(
            kd2_file="path/to/kanjidic2.xml",
            inputs=[{
                "type": "stream",
                "encoding": "utf8",
                "separator": "\\n",
                "path": "path/to/kanjis.txt"
            }],
            outputs=[{
                "type": "stream",
                "indent": 4,
                "path": "path/to/db.json"
            }]
        )

    :param inputs: input streams
    :param output: output streams
    :param kwargs: context
    :return: a JSON object with kanjis data
    """
    db = kwargs[input["name"]]

    for kanji, data in db.items():
        cp = encoding.get_codepoint(kanji)
        if len(cp) < 5:
            cp = "0{}".format(cp)
        filename = "{}.svg".format(cp)

        data["vectorial"] = (
            filename
            if os.path.isfile(os.path.join(self.plugin_config["sources"], filename))
            else None
        )



from test.test_builder import *
from test.test_encoding import *
from test.test_loader import *
from test.test_plugins_helloworld import *
from test.test_plugins_kanjidic2 import *
from test.test_plugins_kanjistream import *
from test.test_plugins_jsonstream import *
from test.test_plugins_mergedict import *
from test.test_sample_generatedb import *
from test.test_service import *



# -*- coding: utf-8 -*-
__all__ = ["PluginsKanjiStreamTestCase"]
import os
import unittest
from kanjidb import builder
from kanjidb.builder.plugins import kanjistream
from kanjidb.encoding import UNICODE_PLUS

DATA_DIR = os.path.join(os.path.dirname(__file__), "data")
KANJIS_UTF8_TXT = os.path.join(DATA_DIR, "kanjis_utf8.txt")
KANJIS_UNICODE_TXT = os.path.join(DATA_DIR, "kanjis_unicode.txt")
SAMPLE_TXT = os.path.join(DATA_DIR, "sample_kanjistream.yml")


class PluginsKanjiStreamTestCase(unittest.TestCase):
    def test_plugin(self):
        """Running via Plugin interface.
        """
        plugin = kanjistream.Plugin()
        config = plugin.required_config
        config.update(
            {
                "inputs": [
                    {
                        "type": "stream",
                        "encoding": UNICODE_PLUS,
                        "separator": ";",
                        "path": KANJIS_UNICODE_TXT,
                    }
                ],
                "outputs": [{"type": "var", "name": "kanjis"}],
            }
        )

        plugin.configure(global_config={}, plugin_config=config)

        result = plugin()
        self.assertTrue("kanjis" in result, "Invalid output")
        kanjis = result["kanjis"]
        self.assertEqual(kanjis, ["一", "二"], "Invalid result")

    def test_run(self):
        """Running via code.
        """
        kwargs = {}

        # Read file and store to "kanjis"
        kanjis = kanjistream.run(
            inputs=[
                {
                    "type": "stream",
                    "encoding": UNICODE_PLUS,
                    "separator": ";",
                    "path": KANJIS_UNICODE_TXT,
                }
            ],
            outputs=[{"type": "var", "name": "kanjis"}],
            kwargs=kwargs,
        )

        # Check both kwargs and return value
        self.assertTrue("kanjis" in kwargs, "Invalid output")
        self.assertEqual(kwargs["kanjis"], ["一", "二"], "Invalid result")
        self.assertTrue(kanjis, "Invalid output")
        self.assertEqual(kanjis, ["一", "二"], "Invalid result")

        # Get "kanjis" and write to stdout
        kanjistream.run(
            inputs=[{"type": "var", "name": "kanjis"}],
            outputs=[
                {
                    "type": "stream",
                    "encoding": UNICODE_PLUS,
                    "separator": ";",
                    "path": "-",
                }
            ],
            kwargs=kwargs,
        )

    def test_sample(self):
        builder.main([SAMPLE_TXT])

    def test_loads(self):
        # UTF8 encoded
        self.assertEqual(kanjistream.loads("一"), ["一"])
        self.assertEqual(kanjistream.loads("一二"), ["一", "二"])
        # With custom separator
        self.assertEqual(kanjistream.loads("一;二", sep=";"), ["一", "二"])
        # Unicode, keep same encoding
        self.assertEqual(
            kanjistream.loads("U+4E00;U4E8C", sep=";"), ["U+4E00", "U4E8C"]
        )
        # Unicode, convert to UTF8
        self.assertEqual(
            kanjistream.loads("U+4E00;U4E8C", encoding=UNICODE_PLUS, sep=";"),
            ["一", "二"],
        )

    def test_load(self):
        # UTF8 encoded
        self.assertEqual(kanjistream.load(KANJIS_UTF8_TXT, sep=";"), ["一", "二"])
        # Unicode
        self.assertEqual(
            kanjistream.load(KANJIS_UNICODE_TXT, encoding=UNICODE_PLUS, sep=";"),
            ["一", "二"],
        )

    def test_dump(self):
        kanjistream.dump(["一", "二"], "-", encoding=UNICODE_PLUS, sep=";")


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
__all__ = ["Plugin", "run"]
from kanjidb.builder.plugins import PluginBase


class Plugin(PluginBase):
    @property
    def template_config(self):
        return {
            "inputs": ["first", "second"],
            "output": "result",
        }

    @property
    def required_config(self):
        config = self.template_config

        return config

    def __call__(self, **kwargs):
        run(
            inputs=self.plugin_config["inputs"],
            output=self.plugin_config["output"],
            kwargs=kwargs,
        )

        return kwargs

    def __repr__(self):
        return "MergeDict"


def run(inputs, output=None, *, kwargs=None):
    """Merge multiple JSON dicts to a single one.

    Configuration:

    .. code-block:: yaml

        run:
        - mergedict:
          inputs:
          - string
          output: string

    Merge dicts from "first" and "second" variables to "result":

    .. code-block:: yaml

        run:
        - mergedict:
          inputs:
          - first
          - second
          output: result

    Python equivalent:

    .. code-block:: python

        from kanjidb.builder.plugins import mergedict

        result = mergedict.run(
            inputs=["first", "second"],
            kwargs={"first": {...}, "second": {...}}
        )

    :param inputs: input vars
    :param output: output var
    :param kwargs: context
    :return: merged JSON dict
    """
    kwargs = kwargs if kwargs is not None else {}

    o = {}

    for _ in inputs:
        o.update(kwargs[_])

    if output:
        kwargs[output] = o

    return o



# -*- coding: utf-8 -*-
__all__ = [
    "UNICODE_ESCAPE",
    "UNICODE_PLUS",
    "UTF8",
    "encode",
    "decode",
    "encode_all",
    "decode_all",
    "get_codepoint",
]
import re

UNICODE_ESCAPE = "unicode_escape"
UNICODE_PLUS = "unicode_plus"
UTF8 = "utf8"


def get_codepoint(kanji):
    return kanji.encode("unicode_escape")[2:].decode().lower()


def decode(s, *, encoding=None):
    """Decode a kanji.

    The kanji may be unicode encoded:

    ..code-block:: python

        >> decode("U+4E00", encoding=UNICODE_PLUS)
        "一"

        >> decode("U4E00", encoding=UNICODE_PLUS)
        "一"

        >> decode("\\u4E00", encoding=UNICODE_ESCAPE)
        "一"

    Or UTF-8 encoded:

    ..code-block:: python

        >> decode("一", encoding=UTF8)
        "一"

        >> decode("一")
        "一"

    :param s: kanji to decode
    :param encoding: how kanji is encoded
    :return: decoded kanji
    """
    if encoding == UNICODE_ESCAPE or encoding == UNICODE_PLUS:
        m = re.match("^(?:\\\\[uU]|[uU][+]?)([0-9a-fA-F]+)$", s)
        if not m:
            raise Exception('Invalid unicode string "{}"'.format(s))

        return chr(int(m.group(1).upper(), 16))
    else:
        return s


def encode(s, *, encoding=None, prefix=None):
    """Encode a kanji.

    The kanji may be unicode encoded:

    ..code-block:: python

        >> encode("一", encoding=UNICODE_PLUS)
        "U+4e00"

        >> encode("一", encoding=UNICODE_PLUS, prefix="U")
        "U4e00"

        >> encode("一", encoding=UNICODE_ESCAPE)
        "\u4e00"

    Or UTF-8 encoded:

    ..code-block:: python

        >> encode("一", encoding=UTF8)
        "一"

        >> encode("一")
        "一"

    :param s: kanji to encode
    :return: encoded kanji
    """
    if encoding == UNICODE_PLUS or encoding == UNICODE_ESCAPE:
        prefix = (
            prefix
            if prefix is not None
            else ("U+" if encoding == UNICODE_PLUS else "\\u")
        )

        return "{}{}".format(prefix, get_codepoint(s))

    return s


def decode_all(kanjis, *, encoding=None):
    """Decode a list of kanjis.

    :param kanjis: list of kanjis
    :param encoding: encoding
    :return: list of decoded kanjis
    """
    encoding = encoding if encoding is not None else UTF8

    return [decode(_, encoding=encoding) for _ in kanjis]


def encode_all(kanjis, *, encoding=None):
    """Reencode a list of UTF-8 encoded kanjis with a new encoding.

    :param kanjis: list of kanjis
    :param encoding: new encoding
    :return: list of encoded kanjis
    """
    encoding = encoding if encoding is not None else UTF8

    return [encode(_, encoding=encoding) for _ in kanjis]



__all__ = ["setup_logging", "run", "main"]
import os
import sys
import argparse
import json
from aiohttp import web
import logging
from kanjidb.service.app import Application
from kanjidb.service import configuration


def setup_logging(
    *,
    access_logfile=None,
    access_maxbytes=None,
    access_backupcount=None,
    error_logfile=None,
    error_maxbytes=None,
    error_backupcount=None
):
    """Setup logging handlers.

    This setup two `RotatingFileHandler` for `aiohttp.access` and `aiohttp.server` logs.

    :param access_logfile: path for access logfile or `None`
    :param access_maxbytes: max bytes per access logfile
    :param access_backupcount: max number of access logfile to keep
    :param error_logfile: path for error logfile or `None`
    :param error_maxbytes: max bytes per error logfile
    :param error_backupcount: max number of error logfile to keep
    """
    from logging.handlers import RotatingFileHandler

    if access_logfile:
        logging.getLogger("aiohttp.access").addHandler(
            RotatingFileHandler(
                access_logfile,
                maxBytes=access_maxbytes,
                backupCount=access_backupcount,
            )
        )
    if error_logfile:
        logging.getLogger("aiohttp.server").addHandler(
            RotatingFileHandler(
                error_logfile, maxBytes=error_maxbytes, backupCount=error_backupcount,
            )
        )


def run(*, swagger_yml: str, swagger_url: str, base_url: str, port: int, db):
    app = Application(
        swagger_yml=swagger_yml, swagger_url=swagger_url, base_url=base_url, db=db
    )
    web.run_app(app, port=port)


def main(argv):
    parser = argparse.ArgumentParser(prog="Service", description="Help")
    parser.add_argument("directory", type=str, help="config directory")
    parser.add_argument("-v", "--verbose", action="store_true", help="Verbosity level")
    args = parser.parse_args(args=argv)

    config_dir = args.directory
    if not os.path.isdir(config_dir):
        raise NotADirectoryError(config_dir)

    config = configuration.load(os.path.join(config_dir, "config.cnf"))

    logging.basicConfig(level=logging.INFO)

    setup_logging(
        access_logfile=config["logging"].get("access-logfile", None),
        access_maxbytes=int(config["logging"].get("access-maxbytes", None)),
        access_backupcount=int(config["logging"].get("access-backupcount", None)),
        error_logfile=config["logging"].get("error-logfile", None),
        error_maxbytes=int(config["logging"].get("error-maxbytes", None)),
        error_backupcount=int(config["logging"].get("error-backupcount", None)),
    )

    with open(config["service"]["db-file"], "rb") as f:
        content = f.read()
        db = json.loads(content.decode(), encoding="utf8")

    run(
        swagger_yml=config["service"]["swagger-yml"],
        swagger_url=config["service"]["swagger-url"],
        base_url=config["service"]["base-url"],
        port=int(config["service"]["port"]),
        db=db,
    )


if __name__ == "__main__":
    main(sys.argv)



DEFAULT_LOGGING_MAXBYTES = 1000000
DEFAULT_LOGGING_BACKUPCOUNT = 5
DEFAULT_CONFIG = {
    "service": {"port": 8080, "base-url": "/"},
    "logging": {
        "access-logfile": "",
        "access-maxbytes": DEFAULT_LOGGING_MAXBYTES,
        "access-backupcount": DEFAULT_LOGGING_BACKUPCOUNT,
        "error-logfile": "",
        "error-maxbytes": DEFAULT_LOGGING_MAXBYTES,
        "error-backupcount": DEFAULT_LOGGING_BACKUPCOUNT,
    },
    "ssl": {"certfile": "", "keyfile": ""},
}


def load(path):
    """Load the service configuration from file.

    Returns default parameters overriden by the ones in the configuration file.

    :param path: file to load
    :return: a dict containing loaded configuration
    """
    import configparser

    result = dict(DEFAULT_CONFIG)
    config = configparser.ConfigParser()
    config.read(path)
    for s in config.sections():
        result.setdefault(s, {}).update(config.items(s))

    return result



__all__ = ["help", "main"]
import sys


def help():
    return """
Usage:  kanjidb COMMAND [OPTIONS]

A kanji database accessible via REST API

Options:
  -v, --version            Print version information and quit
  -h, --help               Show this help

Commands:
  build       Build kanji database from sources
  run         Run local server and REST API

Run 'kanjidb COMMAND --help' for more information on a command.

"""


def main(argv=None):
    if argv is None:
        argv = sys.argv

    if isinstance(argv, str):
        import shlex

        argv = shlex.split(argv)
    elif not hasattr(argv, "__iter__"):
        raise Exception("expected a list or string")

    if len(argv) >= 2:
        # python -m kanjidb build [ARGS...]
        if argv[1] == "build":
            from kanjidb import builder

            return builder.main(argv[2:])
        # python -m kanjidb run [ARGS...]
        elif argv[1] == "run":
            from kanjidb import service

            return service.main(argv[2:])

    print(help())


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
__all__ = ["BuilderTestCase"]
import os
import unittest
from kanjidb import builder
from kanjidb.builder.configuration import Configuration

DATA_DIR = os.path.join(os.path.dirname(__file__), "data")
KANJIDIC2_XML = os.path.join(DATA_DIR, "kanjidic2.xml")
KANJIS_UNICODE_TXT = os.path.join(DATA_DIR, "kanjis_unicode.txt")


class BuilderTestCase(unittest.TestCase):
    def test_build(self):
        config = Configuration(verbose=True)

        builder.build(config)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
__all__ = ["Plugin", "run", "load", "dump"]
import sys
import functools
import json
from kanjidb.builder.plugins import PluginBase


class Plugin(PluginBase):
    @property
    def template_config(self):
        return {
            "inputs": [{"type": "stream", "path": "-",}],
            "outputs": [{"type": "var", "name": "result"}],
        }

    @property
    def required_config(self):
        config = self.template_config

        return config

    def __call__(self, **kwargs):
        run(
            inputs=self.plugin_config["inputs"],
            outputs=self.plugin_config["outputs"],
            kwargs=kwargs,
        )

        return kwargs

    def __repr__(self):
        return "JSONStream"


def run(inputs, outputs=None, *, kwargs=None):
    """Read/write JSON objects from/to streams.

    Configuration:

    .. code-block:: yaml

        run:
        - jsonstream:
          inputs:
          - type: stream|var
            [path: string] # stream only
            [name: string] # var only
          outputs:
          - type: stream|var
            [path: string] # stream only
            [indent: int] # stream only
            [name: string] # var only

    Read from stdin and store to `result`:

    .. code-block:: yaml

        run:
        - jsonstream:
          inputs:
          - type: stream
            path: "-"
          outputs:
          - type: var
            name: result

    Python equivalent:

    .. code-block:: python

        from kanjidb.builder.plugins import jsonstream

        result = jsonstream.run(
            inputs=[{
                "type": "stream",
                "path": "-"
            }]
        )

    Get from `result` and write to stdout:

    .. code-block:: yaml

        run:
        - jsonstream:
          inputs:
          - type: var
            name: result
          outputs:
          - type: stream
            indent: 4
            path: "-"

    Python equivalent:

    .. code-block:: python

        from kanjidb.builder.plugins import jsonstream

        jsonstream.run(
            inputs=[{
                "type": "var",
                "name": "result"
            }],
            outputs=[{
                "type": "stream",
                "indent": 4,
                "path": "-"
            }],
            kwargs={"result": {...}}
        )

    :param inputs: input streams
    :param outputs: output streams
    :param kwargs: context
    :return: a JSON object read from input streams
    """
    outputs = outputs if outputs is not None else []
    kwargs = kwargs if kwargs is not None else {}

    o = {}

    # Read from inputs
    for _ in inputs:
        if _["type"] == "stream":
            o.update(load(_["path"]))
        elif _["type"] == "var":
            o.update(kwargs[_["name"]])
        else:
            raise Exception("Invalid input {}".format(_["type"]))

    # Write to outputs
    for _ in outputs:
        if _["type"] == "stream":
            dump(o, _["path"], indent=_.get("indent", 4))
        elif _["type"] == "var":
            kwargs[_["name"]] = o
        else:
            raise Exception("Invalid output {}".format(_["type"]))

    return o


def load(*streams, loads=None):
    loads = loads if loads is not None else json.loads

    def wrapper():
        for stream in streams:
            # stdin
            if stream == "-":
                stream = sys.stdin
            # Filelike object
            if hasattr(stream, "read"):
                content = stream.read()
            # Filename
            else:
                with open(stream, "rb") as f:
                    content = f.read().decode()

            yield from loads(content)

    return list(wrapper())


def dump(o, *streams, sort_keys=None, indent=None, dumps=None):
    indent = indent if indent is not None else 4
    dumps = (
        dumps
        if dumps is not None
        else functools.partial(
            json.dumps, indent=indent, sort_keys=sort_keys, ensure_ascii=False
        )
    )

    content = dumps(o)

    for stream in streams:
        # stdout
        if stream == "-":
            stream = sys.stdout.buffer
        # Filelike object
        if hasattr(stream, "write"):
            stream.write(content.encode())
        # Filename
        else:
            with open(stream, "wb") as f:
                f.write(content.encode())



# -*- coding: utf-8 -*-
__all__ = ["Plugin", "run"]
from kanjidb.builder.plugins import PluginBase, kanjistream
import kanjidb.encoding


class Plugin(PluginBase):
    MESSAGE = ["今", "日", "わ"]
    STDOUT = {"type": "stream", "encoding": kanjidb.encoding.UTF8, "path": "-"}

    @property
    def template_config(self):
        return {}

    @property
    def required_config(self):
        return {"outputs": [Plugin.STDOUT]}

    def __call__(self, **kwargs):
        run(outputs=self.plugin_config["outputs"], kwargs=kwargs)

        return kwargs

    def __repr__(self):
        return "HelloWorld"


def run(outputs=None, *, kwargs=None):
    """Write "今日わ" to streams.

    Configuration:

    .. code-block:: yaml

        run:
        - helloworld:
          outputs:
          - type: stream|var
            [separator: string] # stream only
            [encoding: string] # stream only
            [path: string] # stream only
            [name: string] # var only

    Write to stdout:

    .. code-block:: yaml

        run:
        - helloworld: {}

    Python equivalent:

    .. code-block:: python

        from kanjidb.builder.plugins import helloworld

        helloworld.run()

    Store to `result`:

    .. code-block:: yaml

        run:
        - helloworld:
          outputs:
          - type: var
            name: result

    Python equivalent:

    .. code-block:: python

        from kanjidb.builder.plugins import helloworld

        result = helloworld.run()

    :param outputs: output streams
    :param kwargs: context
    :return: a list containing ["今", "日", "わ"]
    """
    outputs = outputs if outputs is not None else [Plugin.STDOUT]
    kwargs = kwargs if kwargs is not None else {}

    for _ in outputs:
        if _["type"] == "stream":
            _["separator"] = ""

    return kanjistream.run(
        inputs=[{"type": "var", "name": "kanjis"}],
        outputs=outputs,
        kwargs={"kanjis": Plugin.MESSAGE},
    )



# -*- coding: utf-8 -*-

# kanjidb's package version information
__version_major__ = "0.1"
__version__ = "{}a2".format(__version_major__)
__version_long__ = "{}a2".format(__version_major__)
__status__ = "Alpha"
__author__ = "Jeremy Morosi"
__author_email__ = "jeremymorosi@hotmail.com"
__url__ = "https://github.com/Nauja/kanjidb"



import io
from setuptools import find_packages, setup


def readme():
    with open("README.md", "r") as f:
        return f.read()


def read(*filenames, **kwargs):
    """ Read contents of multiple files and join them together """
    encoding = kwargs.get("encoding", "utf-8")
    sep = kwargs.get("sep", "\n")
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)


pkg_info = {}
exec(read("kanjidb/__version__.py"), pkg_info)


setup(
    name="kanjidb",
    version=pkg_info["__version__"],
    author=pkg_info["__author__"],
    author_email=pkg_info["__author_email__"],
    url=pkg_info["__url__"],
    project_urls={
        "Bug Tracker": "https://github.com/Nauja/kanjidb/issues",
        "Source Code": "https://github.com/Nauja/kanjidb/",
    },
    description="Kanji Database",
    long_description=readme(),
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=["tests"]),
    install_requires=[
        "aiohttp",
        "aiohttp_cors",
        "aiohttp_swagger",
        "jamdict >= 0.1a6",
        "pyyaml",
    ],
    test_suite="test",
    tests_require=["nose", "nose-cover3"],
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)



from . import __version__ as version_info
from .__version__ import __version_major__, __version_long__, __version__, __status__


from kanjidb.main import *

__all__ = ["help", "main", "__version__", "version_info"]



# -*- coding: utf-8 -*-
__all__ = ["SampleGenerateDBTestCase"]
import os
import unittest
from kanjidb import builder

DATA_DIR = os.path.join(os.path.dirname(__file__), "data")
SAMPLE_TXT = os.path.join(DATA_DIR, "sample_generatedb.yml")


class SampleGenerateDBTestCase(unittest.TestCase):
    def test_sample(self):
        builder.main([SAMPLE_TXT])


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
__all__ = ["LoaderTestCase"]
import os
import unittest
from kanjidb.encoding import UNICODE_PLUS
from kanjidb import loader

DATA_DIR = os.path.join(os.path.dirname(__file__), "data")
KANJIS_UTF8_TXT = os.path.join(DATA_DIR, "kanjis_utf8.txt")
KANJIS_UNICODE_TXT = os.path.join(DATA_DIR, "kanjis_unicode.txt")


class LoaderTestCase(unittest.TestCase):
    pass


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
__all__ = ["PluginsMergeDictTestCase"]
import os
import unittest
from kanjidb import builder
from kanjidb.builder.plugins import mergedict

DATA_DIR = os.path.join(os.path.dirname(__file__), "data")
SAMPLE_TXT = os.path.join(DATA_DIR, "sample_mergedict.yml")

FIRST = {"a": None}
SECOND = {"a": True}
RESULT = dict(FIRST)
RESULT.update(SECOND)


class PluginsMergeDictTestCase(unittest.TestCase):
    def test_plugin(self):
        """Running via Plugin interface.
        """
        plugin = mergedict.Plugin()
        config = plugin.required_config
        config.update(
            {"inputs": ["first", "second"], "output": "result",}
        )

        plugin.configure(global_config={}, plugin_config=config)

        result = plugin(first=FIRST, second=SECOND)
        self.assertTrue("result" in result, "Invalid output")
        self.assertEqual(result["result"], RESULT, "Invalid result")

    def test_run(self):
        """Running via code.
        """
        result = mergedict.run(
            inputs=["first", "second"], kwargs={"first": FIRST, "second": SECOND},
        )

        self.assertTrue(result, "Invalid output")
        self.assertEqual(result, RESULT, "Invalid result")

    def test_sample(self):
        builder.main([SAMPLE_TXT])


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
__all__ = ["ServiceTestCase"]
import os
import unittest
import json
from aiohttp.test_utils import AioHTTPTestCase, unittest_run_loop
from kanjidb.service import configuration, Application

DATA_DIR = os.path.join(os.path.dirname(__file__), "data")
CONFIG_CNF = os.path.join(DATA_DIR, "config.cnf")


class ServiceTestCase(AioHTTPTestCase):
    async def get_application(self):
        config = configuration.load(CONFIG_CNF)

        with open(config["service"]["db-file"], "rb") as f:
            content = f.read()
            db = json.loads(content.decode(), encoding="utf8")

        return Application(
            swagger_yml=config["service"]["swagger-yml"],
            swagger_url=config["service"]["swagger-url"],
            base_url=config["service"]["base-url"],
            db=db,
        )

    @unittest_run_loop
    async def test_doc(self):
        resp = await self.client.request("GET", "/api/v1/doc")
        assert resp.status == 200

    @unittest_run_loop
    async def test_api(self):
        resp = await self.client.request("GET", "/api/v1/kanji")
        assert resp.status == 200
        resp = await self.client.request("GET", "/api/v1/kanji/一")
        assert resp.status == 200


if __name__ == "__main__":
    unittest.main()
